/**
  \file ReservoirFunc.cpp
 */


/*
<func2doc>

</func2doc>
 */


#include <openfluid/base.hpp>
#include <openfluid/core.hpp>


// =====================================================================
// =====================================================================


DECLARE_PLUGIN_HOOKS;


// =====================================================================
// =====================================================================


BEGIN_SIGNATURE_HOOK
DECLARE_SIGNATURE_ID("water.surf-uz.flow-su.reservoir-c");
DECLARE_SIGNATURE_NAME("water storage modelling");
DECLARE_SIGNATURE_DESCRIPTION("this fonction distributes a water height in the different layers of the unsaturated zone");

DECLARE_SIGNATURE_VERSION("11.05");
DECLARE_SIGNATURE_SDKVERSION;
DECLARE_SIGNATURE_STATUS(openfluid::base::EXPERIMENTAL);

DECLARE_SIGNATURE_DOMAIN("unsaturated zone");
DECLARE_SIGNATURE_PROCESS("soil water flow");
DECLARE_SIGNATURE_METHOD("capacitive modelling");
DECLARE_SIGNATURE_AUTHORNAME("David Crevoisier and Cécile Dagès");
DECLARE_SIGNATURE_AUTHOREMAIL("david.crevoisier@supagro.inra.fr and cecile.dages@supagro.inra.fr");


// Parameters
DECLARE_FUNCTION_PARAM("nCouche","Number of soil layers","-")



// Input data
DECLARE_REQUIRED_INPUTDATA("th_cc","SU","Field capacity","m3/m3")


// Variables
DECLARE_PRODUCED_VAR("water.surf-uz.theta[]","SU","Soil moisture profile","m3/m3")
DECLARE_PRODUCED_VAR("water.surf-uz.drainage","SU","Drainage below soil profile","m")
DECLARE_PRODUCED_VAR("water.surf-uz.storage","SU","Water storage in the whole soil column","m")
DECLARE_REQUIRED_VAR("water.atm-surf.H.rain","SU","Produced variable, distributed on SU - rainfall height on each SU by time step","m")


END_SIGNATURE_HOOK


// =====================================================================
// =====================================================================


/**

 */
class ReservoirFunction : public openfluid::base::PluggableFunction
{
private:

	int m_nCouche;
	// IDVectorValueMap : vecteurs spatialises
	openfluid::core::IDScalarValueMap m_th_cc;
	openfluid::core::IDVectorValueMap m_thCI;
	openfluid::core::IDVectorValueMap m_dz;


public:


	ReservoirFunction(): PluggableFunction()
	{


	}


	// =====================================================================
	// =====================================================================


	~ReservoirFunction()
	{


	}


	// =====================================================================
	// =====================================================================


	bool initParams(openfluid::core::FuncParamsMap_t Params)
	{

		OPENFLUID_GetFunctionParameter (Params,("nCouche"),&m_nCouche);
		return true;
	}

	// =====================================================================
	// =====================================================================


	bool prepareData()
	{


		return true;
	}


	// =====================================================================
	// =====================================================================


	bool checkConsistency()
	{


		return true;
	}


	// =====================================================================
	// =====================================================================


	bool initializeRun(const openfluid::base::SimulationInfo* /*SimInfo*/)
	{


		openfluid::core::UnitID_t ID;
		openfluid::core::Unit *SU;
		double th_cc,thetai1,thetai2,thetai3,thetai4,thetai5;
		double dz1,dz2,dz3,dz4,dz5;
		DECLARE_UNITS_ORDERED_LOOP(1) ;


		BEGIN_UNITS_ORDERED_LOOP(1,"SU",SU)
		OPENFLUID_GetInputData(SU,"th_cc",&th_cc);
		OPENFLUID_GetInputData(SU,"thetai1",&thetai1);
		OPENFLUID_GetInputData(SU,"thetai2",&thetai2);
		OPENFLUID_GetInputData(SU,"thetai3",&thetai3);
		OPENFLUID_GetInputData(SU,"thetai4",&thetai4);
		OPENFLUID_GetInputData(SU,"thetai5",&thetai5);
		OPENFLUID_GetInputData(SU,"dz1",&dz1);
		OPENFLUID_GetInputData(SU,"dz2",&dz2);
		OPENFLUID_GetInputData(SU,"dz3",&dz3);
		OPENFLUID_GetInputData(SU,"dz4",&dz4);
		OPENFLUID_GetInputData(SU,"dz5",&dz5);

		ID = SU->getID();
		// creation des vecteurs déclarés comme attributs
		m_dz[ID] = openfluid::core::VectorValue(m_nCouche);
		m_thCI[ID] = openfluid::core::VectorValue(m_nCouche);

		m_th_cc[ID]  =th_cc;
		m_thCI[ID][0]=thetai1;
		m_thCI[ID][1]=thetai2;
		m_thCI[ID][2]=thetai3;
		m_thCI[ID][3]=thetai4;
		m_thCI[ID][4]=thetai5;

		m_dz[ID][0]=dz1;
		m_dz[ID][1]=dz2;
		m_dz[ID][2]=dz3;
		m_dz[ID][3]=dz4;
		m_dz[ID][4]=dz5;

		END_LOOP;

		return true;
	}

	// =====================================================================
	// =====================================================================


	bool runStep(const openfluid::base::SimulationStatus* SimStatus)
	{

		openfluid::core::ScalarValue TmpRain;
		openfluid::core::ScalarValue drainage;
		openfluid::core::ScalarValue storage;
		openfluid::core::UnitID_t ID;
		int CurrentStep;
		int PreviousStep;
		int k;
		openfluid::core::VectorValue Theta;
		openfluid::core::Unit *SU;
		DECLARE_UNITS_ORDERED_LOOP(1) ;

		CurrentStep  = SimStatus->getCurrentStep();
		PreviousStep = SimStatus->getCurrentStep()-1;

		BEGIN_UNITS_ORDERED_LOOP(1,"SU",SU)

		ID = SU->getID();

		OPENFLUID_GetVariable(SU,("water.atm-surf.H.rain"),CurrentStep,&TmpRain);
		if (PreviousStep==-1)
			drainage = 0;
		else
			OPENFLUID_GetVariable(SU,("water.surf-uz.drainage"),PreviousStep,&drainage);

		Theta = openfluid::core::VectorValue(m_nCouche);

		double apportActu = TmpRain;
		for (int i=0;i<m_nCouche;i++) {
			if (apportActu>=(m_th_cc[ID]-m_thCI[ID][i]) * m_dz[ID][i]) {
				Theta[i]   = m_th_cc[ID];
				apportActu = apportActu - (m_th_cc[ID]-m_thCI[ID][i])*m_dz[ID][i];
			}
			else {
				Theta[i]   = m_thCI[ID][i] + apportActu/m_dz[ID][i];
				apportActu = 0.0;
			}
		}
		drainage += apportActu;

		/*mise à jour des valeurs de th ini pour le pas de temps suivant
		 * avec les valeurs de th du pas de temps en cours
		 */

		storage = 0;
		for (k=0;k<m_nCouche;k++)
		{
			m_thCI[ID][k] = Theta[k];
			storage      += Theta[k]*m_dz[ID][k];
		}

		OPENFLUID_AppendVariable(SU,"water.surf-uz.theta",Theta);
		OPENFLUID_AppendVariable(SU,"water.surf-uz.storage",storage);
		OPENFLUID_AppendVariable(SU,"water.surf-uz.drainage",drainage);

		// destruction des vecteurs
		Theta.clear();
		END_LOOP;

		return true;
	}

	// =====================================================================
	// =====================================================================


	bool finalizeRun(const openfluid::base::SimulationInfo* /*SimInfo*/)
	{


		return true;
	}

};


// =====================================================================
// =====================================================================


DEFINE_FUNCTION_HOOK(ReservoirFunction);

